@@ -4,24 +4,26 @@ General tools useful to create NeXus format files.
 
 __author__ = "Diamond Light Source - Scientific Software"
 __email__ = "scientificsoftware@diamond.ac.uk"
-__version__ = "0.5.4"
+__version__ = "0.5.5"
 __version_tuple__ = tuple(int(x) for x in __version__.split("."))
 
 import re
 import sys
+import h5py
 import pint
 import numpy as np
 
 from pathlib import Path
 from datetime import datetime
-from typing import Optional, List
+from typing import Any, Optional, List, Union
 
 # Initialize registry and a Quantity constructor
 ureg = pint.UnitRegistry()
 Q_ = ureg.Quantity
 
-# Filename pattern: filename_######.h5
-P = re.compile(r"(.*)_(?:\d+)")
+# Filename pattern: filename_######.h5 or filename_meta.h5
+# P = re.compile(r"(.*)_(?:\d+)")
+P = re.compile(r"(.*)_(?:meta|\d+)")
 
 # Format strings for timestamps
 format_list = ["%Y-%m-%dT%H:%M:%S", "%Y-%m-%d %H:%M:%S", "%a %b %d %Y %H:%M:%S"]
@@ -41,42 +43,62 @@ def imgcif2mcstas(vector):
     return tuple(np.dot(c2n, vector))
 
 
-def get_filename_template(master_filename: Path) -> str:
+def get_filename_template(input_filename: Path) -> str:
     """
-    Get the data file name template from the master file.
+    Get the data file name template from either the master or the meta file.
 
     Args:
-        master_filename:    Path object containing the name of master file.
-                            The format should be either file_master.h5 or file.nxs.
+        master_filename:    Path object containing the name of master or meta file.
+                            The format should be either file_master.h5, file.nxs for a master file,
+                            file_meta.h5 for a meta file.
     Returns:
         filename_template:  String template for the name of blank data file.
     """
-    if master_filename.suffix == ".nxs":
-        filename_root = master_filename.stem
-        filename_template = master_filename.parent / f"{filename_root}_%0{6}d.h5"
-    elif master_filename.suffix == ".h5":
-        filename = master_filename.stem.replace("master", f"%0{6}d")
-        filename_template = master_filename.parent / f"{filename}.h5"
+    if input_filename.suffix == ".nxs":
+        filename_root = input_filename.stem
+        filename_template = input_filename.parent / f"{filename_root}_%0{6}d.h5"
+    elif input_filename.suffix == ".h5" and "master" in input_filename.as_posix():
+        filename = input_filename.stem.replace("master", f"%0{6}d")
+        filename_template = input_filename.parent / f"{filename}.h5"
+    elif input_filename.suffix == ".h5" and "meta" in input_filename.as_posix():
+        filename = input_filename.stem.replace("meta", f"%0{6}d")
+        filename_template = input_filename.parent / f"{filename}.h5"
     else:
-        sys.exit("Master file did not have the expected format.")
+        sys.exit(
+            "Input file did not have the expected format for a master or meta file."
+        )
     # so that filename_template.as_posix() % 1 will become filename_000001.h5
     return filename_template.as_posix()
 
 
-def get_nexus_filename(data_filename: Path) -> Path:
+def get_nexus_filename(input_filename: Path) -> Path:
     """
     Get the filename for the NeXus file from the stem of the input file name, by .
 
     Args:
-        data_filename:  Input file path.
+        input_filename:  File name and path of either a .h5 data file or a _meta.h5 file.
     Returns:
         NeXus file name (.nxs) path.
     """
-    filename_stem = P.fullmatch(data_filename.stem)[1]
-    nxs_filename = data_filename.parent / f"{filename_stem}.nxs"
+    filename_stem = P.fullmatch(input_filename.stem)[1]
+    nxs_filename = input_filename.parent / f"{filename_stem}.nxs"
     return nxs_filename
 
 
+def walk_nxs(nxs_obj: Union[h5py.File, h5py.Group]) -> List[str]:
+    """
+    Walk all the groups, subgroups and datasets of an object.
+
+    Args:
+        nxs_obj:    Object to walk through, could be a file or a group.
+    Returns:
+        obj_list:   List of strings.
+    """
+    obj_list = []
+    nxs_obj.visit(obj_list.append)
+    return obj_list
+
+
 def split_arrays(coord_frame: str, axes_names: List, array: List) -> dict:
     """
     Split a list of values into arrays.
@@ -127,20 +149,24 @@ def get_iso_timestamp(ts: str) -> str:
     return ts_iso
 
 
-def units_of_length(q: str, to_base: Optional[bool] = False) -> pint.Quantity:
+def units_of_length(q: Any, to_base: Optional[bool] = False) -> pint.Quantity:
     """
-    Check that a quantity of length is compatible with NX_LENGTH, defaulting to mm if dimensionless.
+    Check that a quantity of length is compatible with NX_LENGTH, defaulting to m if dimensionless.
 
     Args:
-        q:          A string that can be interpreted as a pint Quantity, it can be dimensionless.
+        q:          An object that can be interpreted as a pint Quantity, it can be dimensionless.
         to_base:    If True, convert to base units of length (m).
     Returns:
         quantity:   A pint quantity with units applied if it was dimensionless.
     """
     quantity = Q_(q)
-    if quantity <= 0:
-        raise ValueError("Quantity must be positive")
-    quantity = quantity * ureg.mm if quantity.dimensionless else quantity
+    try:
+        if any(quantity) <= 0:
+            raise ValueError("Quantity (length) must be positive.")
+    except TypeError:  # FIXME gorilla here
+        if quantity <= 0:
+            raise ValueError("Quantity (length) must be positive.")
+    quantity = quantity * ureg.m if quantity.dimensionless else quantity
     if quantity.check("[length]"):
         if to_base is True:
             return quantity.to_base_units()
@@ -164,7 +190,7 @@ def units_of_time(q: str) -> pint.Quantity:
     """
     quantity = Q_(q)
     if quantity <= 0:
-        raise ValueError("Quantity of time must be positive")
+        raise ValueError("Quantity (time) of time must be positive.")
     quantity = quantity * ureg.s if quantity.dimensionless else quantity
     if quantity.check("[time]"):
         return quantity.to_base_units()
