@@ -66,9 +66,15 @@ detector_scope = freephil.parse(
       flatfield = None
         .type = path
         .help = "If path is given, add flatfield correction data field"
+      flatfield_applied = False
+        .type = bool
+        .help = "If the flatfield location is known, specify whether it's been applied of not."
       pixel_mask = None
         .type = path
         .help = "if path is given, add link to bad pixel mask"
+      pixel_mask_applied = False
+        .type = bool
+        .help = "If the mask location is known, specify whether it's been applied of not."
       image_size = 4362 4148
         .type = ints(size = 2)
         .help = "Image size in pixels: (slow, fast)"
@@ -99,9 +105,9 @@ detector_scope = freephil.parse(
       units = deg mm
         .type = strings
         .help = "Axis units, from mm or deg"
-      software_version = 1.1.3
+      software_version = None
         .type = str
-        .help = "Tristan specific - Software version"
+        .help = "Detector software version"
       detector_tick = 1562.5ps
         .type = str
         .help = "Tristan specific - detector tick, in ps"
